/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                    CHIPCON CC1010 EXAMPLE PROGRAM            *
 *      ***   + +   ***            RF232 (RS232 <-> RF <-> RS232)            *
 *      ***   +++   ***              Wireless null-modem cable               *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * This program demonstrates the use of the SPP library, timers, and the     *
 * serial port. The program is mainly interrupt driven.                      *
 *                                                                           *
 * A pair of CC1010EBs running this program will function as a wireless      *
 * null-modem cable, between serial port 0 on each board.                    *
 *                                                                           *
 *                                                                           *
 * +----+            +--------+                 +--------+            +----+ *
 * |    |  UART0 RX  |        |                 |        |  UART0 RX  |    | *
 * |    |----------->|        | RF/SPP with ACK |        |<-----------|    | *
 * | PC |            | CC1010 |<--------------->| CC1010 |            | PC | *
 * |    |  UART0 TX  |        |   Half duplex   |        |  UART0 TX  |    | *
 * |    |<-----------|        |                 |        |----------->|    | *
 * +----+            +--------+                 +--------+            +----+ *
 *                                                                           *
 *        38400 baud            RF @ 19200 baud            38400 baud        *
 *                                                                           *
 *                                                                           *
 * FEATURES:                                                                 *
 *     - Max RS232 baud rate: 38400 (no parity bit)                          *
 *     - Max RF baud rate: 19200/NRZ                                         * 
 *     - Half-duplex                                                         *
 *     - Hardware flow control (CTS only)                                    *
 *     - Automatic retransmission and transmission crash recovery through    *
 *       randomized timeouts                                                 *
 *     - Serial input/output while the transceiver is active (through use of *
 *       interrupts)                                                         *
 *     - Automatic recalibration of the transceiver (every 2 minutes)        *
 *                                                                           *
 * EXPECTED PERFORMANCE (serial port data transfer):                         *
 *     - 1k Xmodem: >7000 bps with RF@9600 and UART@38400                    *
 *     - 1k Xmodem: >11500 bps with RF@19200 and UART@38400                  *
 *                                                                           *
 * LED INDICATORS:                                                           *
 *     - Red:    Waiting for retransmission                                  *
 *     - Yellow: Transmitting                                                *
 *     - Green:  Toggled for each received packet                            *
 *     - Blue:   Hardware flow control activated (CTS)                       *
 *                                                                           *
 * HOW IT WORKS:                                                             *
 * The main loop takes care of RF transmission and reception.                *
 * The UART0 ISR takes care of UART transmission and reception.              *
 *                                                                           *
 * RF->UART:                                                                 *
 *     When an RF packet has been received, the RF RX and UART TX buffer     *
 *     are switched, and the packet data is transmitted over the UART        *
 *     (after making sure that it's ready).                                  *
 *                                                                           *
 * UART->RF:                                                                 *
 *     When the first byte is received, a start RF countdown begins. To      *
 *     avoid buffer overruns the serial port flow control is enabled         *
 *     after TX_BUFFER_LIMIT bytes. The RF TX buffer and the UART RX         *
 *     buffers are switched, and the RF transmission begins. In the mean     *
 *     while the UART is still ready for receptions (into the other          *
 *     buffer).                                                              *
 *****************************************************************************
 * Author:              JOL                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 * 1.0  2002/07/18      First public release                                 *
 *                                                                           *
 * $Log: rf232.c,v $
 * Revision 1.5  2003/04/03 12:27:36  tos
 * Bugfix:
 * Disabled timer 3 interrupt in UART0_ISR (Caused the program to hang on TX_BUFFER_LIMIT)
 * Removed SPP timer function calls after recalibration (sppSetupRF).
 *
 * Revision 1.4  2003/02/03 08:25:33  tos
 * Correction of UART flow contol: activate flow control to avoid buffer overrun.
 *
 * Revision 1.3  2003/01/30 07:29:19  tos
 * Corrected recalibration procedure: re-initialization of function pointers (Call Back).
 *
 * Revision 1.2  2002/11/20 14:10:00  kht
 * Added startup macros
 * Moved call to sppSetupRF to be called before setting callbacks, this was
 * required due to change in SPP library
 *
 * Revision 1.1  2002/10/14 10:55:46  tos
 * Initial version in CVS.
 *
 *                                                                           *
 ****************************************************************************/

#include <chipcon/reg1010.h>
#include <chipcon/cc1010eb.h>
#include <chipcon/hal.h>
#include <chipcon/cul.h>
#include <stdlib.h>     // srand(), rand()

// MACRO: Switch two values without a third temporary variable
#define SWITCH(a,b) do { b^=a; a^=b; b^=a; } while (0)


// Buffer sizes
#define RX_BUFFER_LENGTH    255
#define TX_BUFFER_LENGTH    255

// Address definitions
#define DESTINATION_ADDRESS 255
#define MY_ADDRESS          255

// UART0 flow control
#define UART0_FLOW_CONTROL  (P1_5)
#define STOP                1
#define READY               0
#define TX_BUFFER_LIMIT     150

// Data buffers
byte xdata pRXBuffer[2][RX_BUFFER_LENGTH]; byte rxDataLen[2]; // RF RX data buffer, valid length (set by RF RX)
byte xdata pTXBuffer[2][TX_BUFFER_LENGTH]; byte txDataLen[2]; // RF TX data buffer, valid length (set by UART RX)
byte xdata uartRXBuffer, uartTXBuffer, uartRXPos, uartTXPos, rfRXBuffer, rfTXBuffer; 

// Transmission request flag
byte xdata txRequest;
#define TX_REQUEST_ON       1
#define TX_REQUEST_OFF      0

// Transmission timeout
word xdata startTXTimeout;
#define START_TX_TIMEOUT    7
#define SHORT_TX_TIMEOUT    2

// Recalibration request flag
byte xdata recalibRequest;
#define RECALIB_REQUEST_ON  1
#define RECALIB_REQUEST_OFF 0

// Recalibration timeout
word xdata recalibTimeout;
#define RECALIB_TIMEOUT     12000 // Every 2 minutes

// Reception timeout
#define NORMAL_TIMEOUT      100
#define RETRY_TX_TIMEOUT    (1 + (rand() & 0x001F))


// Used to detect and ignore retransmissions (SPP_SEQUENCE_BIT)
byte xdata lastRXflags;

// SPP variables
SPP_SETTINGS xdata sppSettings;
SPP_RX_INFO xdata RXI;
SPP_TX_INFO xdata TXI;

// Function prototypes
void startTX (void);
void startRecalib (void);



//----------------------------------------------------------------------------
//  MAIN PROGRAM
//----------------------------------------------------------------------------
void main() {

#ifdef FREQ868
	// X-tal frequency: 14.745600 MHz
	// RF frequency A: 868.277200 MHz	Rx
	// RF frequency B: 868.277200 MHz	Tx
	// RX Mode: Low side LO
	// Frequency separation: 64 kHz
	// Data rate: 19.2 kBaud
	// Data Format: NRZ
	// RF output power: 4 dBm
	// IF/RSSI: RSSI Enabled
	
    RF_RXTXPAIR_SETTINGS code RF_SETTINGS = {
        0xA3, 0x27, 0x11,    // Modem 0, 1 and 2
        0x75, 0xA0, 0x00,    // Freq A
        0x58, 0x32, 0x8D,    // Freq B
        0x01, 0xAB,          // FSEP 1 and 0
        0x40,                // PLL_RX
        0x30,                // PLL_TX
        0x6C,                // CURRENT_RX
        0xF3,                // CURRENT_TX
        0x32,                // FREND
        0xFF,                // PA_POW
        0x00,                // MATCH
        0x00,                // PRESCALER
    };
#endif

#ifdef FREQ915
    // X-tal frequency: 14.745600 MHz
    // RF frequency A: 915.027455 MHz	Rx
    // RF frequency B: 915.027455 MHz	Tx
    // RX Mode: Low side LO
    // Frequency separation: 64 kHz
    // Data rate: 19.2 kBaud
    // Data Format: NRZ
    // RF output power: 4 dBm
    // IF/RSSI: RSSI Enabled

    RF_RXTXPAIR_SETTINGS code RF_SETTINGS = {
        0xA3, 0x27, 0x11,    // Modem 0, 1 and 2
        0xAA, 0x80, 0x00,    // Freq A
        0x5C, 0xF4, 0x02,    // Freq B
        0x01, 0xAB,          // FSEP 1 and 0
        0x58,                // PLL_RX
        0x30,                // PLL_TX
        0x6C,                // CURRENT_RX
        0xF3,                // CURRENT_TX
        0x32,                // FREND
        0xFF,                // PA_POW
        0x00,                // MATCH
        0x00,                // PRESCALER
    };
#endif

#ifdef FREQ433
    // X-tal frequency: 14.745600 MHz
    // RF frequency A: 433.302000 MHz	Rx
    // RF frequency B: 433.302000 MHz	Tx
    // RX Mode: Low side LO
    // Frequency separation: 64 kHz
    // Data rate: 19.2 kBaud
    // Data Format: NRZ
    // RF output power: 10 dBm
    // IF/RSSI: RSSI Enabled

    RF_RXTXPAIR_SETTINGS code RF_SETTINGS = {
        0xA3, 0x2F, 0x0E,    // Modem 0, 1 and 2
        0x58, 0x00, 0x00,    // Freq A
        0x41, 0xFC, 0x9C,    // Freq B
        0x02, 0x80,          // FSEP 1 and 0
        0x60,                // PLL_RX
        0x48,                // PLL_TX
        0x44,                // CURRENT_RX
        0x81,                // CURRENT_TX
        0x0A,                // FREND
        0xFF,                // PA_POW
        0xC0,                // MATCH
        0x00,                // PRESCALER
    };
#endif

    // Calibration data
    RF_RXTXPAIR_CALDATA xdata RF_CALDATA;

    // Initialize peripherals
    WDT_ENABLE(FALSE);
    RLED_OE(TRUE); RLED = LED_OFF;
    YLED_OE(TRUE); YLED = LED_OFF;
    GLED_OE(TRUE); GLED = LED_ON;
    BLED_OE(TRUE); BLED = LED_OFF;

    // Set optimum settings for speed and low power consumption
    MEM_NO_WAIT_STATES();
    FLASH_SET_POWER_MODE(FLASH_STANDBY_BETWEEN_READS);

    // Local settings
    sppSettings.myAddress = MY_ADDRESS;
    sppSettings.rxTimeout = NORMAL_TIMEOUT;
    sppSettings.txAckTimeout = 2;
    sppSettings.txAttempts = 2;
    sppSettings.txPreambleByteCount = 7;

    // Setup serial port flow control
    PORTDIRBIT(1, 5, POUT);
    UART0_FLOW_CONTROL = READY;

    // Initialize the buffer state variables
    uartRXBuffer = 1;
    uartTXBuffer = 1;
    uartRXPos = 0;
    uartTXPos = 0;
    rfRXBuffer = 0;
    rfTXBuffer = 0;

    // RF setup
    sppSetupRF(&RF_SETTINGS, &RF_CALDATA, TRUE);

    // Initialize the SPP timer
    sppStartTimer(CC1010EB_CLKFREQ);
    SPP_INIT_TIMEOUTS();

    // Initialize the start TX timeout
    txRequest = TX_REQUEST_OFF;
    startTXTimeout = 0;
    sppSetTimerCB (SPP_CUSTOM_0_TIMER, startTX, &startTXTimeout);

    // Initialize the recalibration timeout
    recalibRequest = RECALIB_REQUEST_OFF;
    recalibTimeout = RECALIB_TIMEOUT;
    sppSetTimerCB (SPP_CUSTOM_1_TIMER, startRecalib, &recalibTimeout);

    // Initialize the RF transmit structure
    TXI.destination = DESTINATION_ADDRESS;
    TXI.pDataBuffer = pTXBuffer[rfTXBuffer];
    TXI.flags = SPP_ACK_REQ;
    TXI.status = SPP_TX_FINISHED; // Nothing has gone wrong yet...

    // Initialize the RF receive structure
    RXI.maxDataLen = RX_BUFFER_LENGTH;
    RXI.pDataBuffer = pRXBuffer[rfRXBuffer];
    lastRXflags = TXI.flags | SPP_SEQUENCE_BIT;

    // Initialize the random number generator (used to generate random timeouts)
    srand(0x1234);


    // Serial port 0 setup
    UART0_SETUP(38400, CC1010EB_CLKFREQ, UART_NO_PARITY | UART_RX_TX | UART_ISR);

    // Run forever
    while (TRUE) {
        
        // Receive a packet
        if (sppReceive(&RXI) == SPP_RX_STARTED) {

            // Wait for RF RX to complete
            do {
                if (txRequest && (TXI.status == SPP_TX_FINISHED) && (RXI.status == SPP_RX_WAITING)) {
                    sppReset();
                }
            } while (SPP_STATUS() != SPP_IDLE_MODE); 

            // Check the results (ignore retransmitted messages)
            if (RXI.status == SPP_RX_FINISHED) {
                if (SPP_SEQUENCE_BIT & (lastRXflags ^ RXI.flags)) {

                    // Wait for UART TX to complete 
                    while (uartRXPos != rxDataLen[uartRXBuffer]);
                                                                            //
                    // Only process packets with payload data               //
                    if ((rxDataLen[rfRXBuffer] = RXI.dataLen) != 0) {       //
                                                                            //
                        // Switch buffers and initiate UART0 transmission	// No UART TX interrupts in this section
                        GLED = ~GLED;                                       //
                        SWITCH(uartRXBuffer, rfRXBuffer);                   //
                        RXI.pDataBuffer = pRXBuffer[rfRXBuffer];            //
                        uartRXPos = 0;                                      //
                        UART0_SEND(pRXBuffer[uartRXBuffer][uartRXPos]);
                    }
                }
                lastRXflags = RXI.flags;
            }
        }

        // Transmit packet, if requested
        if (txRequest) {
            if (sppSend(&TXI) == SPP_TX_STARTED) {

                // Wait for the transmission to finish
                YLED = LED_ON;
                while (SPP_STATUS() != SPP_IDLE_MODE);
                YLED = LED_OFF;

                // Check out the results
                if (TXI.status == SPP_TX_FINISHED) {
                    sppSettings.rxTimeout = NORMAL_TIMEOUT;
                    RLED = LED_OFF;
                    txRequest = TX_REQUEST_OFF;
                } else {
                    RLED = LED_ON;
                    sppSettings.rxTimeout = RETRY_TX_TIMEOUT;
                }
            }
        }

        // Recalibrate the transceiver if requested to
        if (recalibRequest) {
            sppSetupRF(&RF_SETTINGS, &RF_CALDATA, TRUE);
        }
    }
} // main




//----------------------------------------------------------------------------
//  Start transmission (called by TIMER3_ISR(), in sppTimer.c)
//----------------------------------------------------------------------------
void startTX() {

    if (!txRequest) {
        SWITCH(uartTXBuffer, rfTXBuffer);
        TXI.pDataBuffer = pTXBuffer[rfTXBuffer];
        TXI.dataLen = uartTXPos;
        uartTXPos = 0;
        UART0_FLOW_CONTROL = READY; BLED = LED_OFF;
        txRequest = TX_REQUEST_ON;
    } else {
        startTXTimeout = SHORT_TX_TIMEOUT;
    }
} // startTX





//----------------------------------------------------------------------------
//  Recalibrate the transceiver (called by TIMER3_ISR(), in sppTimer.c)
//----------------------------------------------------------------------------
void startRecalib (void) {
    recalibRequest = RECALIB_REQUEST_ON;
    recalibTimeout = RECALIB_TIMEOUT;
} // recalibrateRF



//----------------------------------------------------------------------------
//  UART 0 interrupt service routine:
//----------------------------------------------------------------------------
void UART0_ISR () interrupt INUM_UART0 {
                  

    // Byte sent (RF RX)
    if (TI_0) {
        INT_SETFLAG (INUM_UART0_TX, INT_CLR);

        // Push bytes until the buffer is empty
        if (++uartRXPos < rxDataLen[uartRXBuffer]) {
            UART0_SEND(pRXBuffer[uartRXBuffer][uartRXPos]);
        } else {
            uartRXPos = rxDataLen[uartRXBuffer];
        }
    }

    // Byte received (RF TX)
    if (RI_0) {
        INT_SETFLAG (INUM_UART0_RX, INT_CLR);
        if (uartTXPos < TX_BUFFER_LENGTH) {
            pTXBuffer[uartTXBuffer][uartTXPos++] = UART0_RECEIVE();

            // Schedule a start TX callback when the first data byte arrives through the UART
            INT_ENABLE (INUM_TIMER3, INT_OFF);
            if (!startTXTimeout) {
                startTXTimeout = START_TX_TIMEOUT;
            } else {
                if (uartTXPos == TX_BUFFER_LIMIT) {
                    // Prevent buffer overrun by turning CTS off (RTS at our end of the cable)
                    UART0_FLOW_CONTROL = STOP; BLED = LED_ON;
                    startTXTimeout = SHORT_TX_TIMEOUT;
                }
            }
            INT_ENABLE (INUM_TIMER3, INT_ON);
        }
    }

} // UART0_ISR

